"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.genAPISDKByPath = void 0;
const tslib_1 = require("tslib");
const path = require("path");
const child_process_1 = require("child_process");
const util_1 = require("../util");
/** 根据路径生成APISDK */
function genAPISDKByPath(ctrlDir, filter = [/^\/api\//g], config, appConfig, fork = child_process_1.fork) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, _reject) => {
            const files = [];
            [].concat(ctrlDir).forEach(dir => {
                files.push(...util_1.getDirFiles(dir));
            });
            if (config.hook) {
                Object.keys(config.hook).forEach(key => {
                    if (typeof config.hook[key] === 'function') {
                        config.hook[key] = config.hook[key].toString();
                    }
                });
            }
            const p = fork(path.join(__dirname, 'gen_sdk_script.js'), null, {
                stdio: 'inherit',
                encoding: 'utf8',
            });
            p.on('message', _data => {
                p.send({
                    files,
                    config,
                    filter: filter.map(f => f.toString()),
                    appConfig,
                });
            });
            p.on('exit', () => {
                resolve();
            });
        });
    });
}
exports.genAPISDKByPath = genAPISDKByPath;
